/* tcpside.c: TCP routines for tcpusb.

   Copyright 2005 Ton van Overbeek.

   This is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.  */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <string.h>
#include <errno.h>

static struct sockaddr_in clientname;

static void
init_sockaddr (struct sockaddr_in *name,
               const char *hostname,
               uint16_t port)
{
  struct hostent *hostinfo;

  memset (name, 0, sizeof (struct sockaddr_in));
  name->sin_family = AF_INET;
  name->sin_port = htons (port);
  hostinfo = gethostbyname (hostname);
  if (hostinfo == NULL) {
    fprintf (stderr, "Unknown host %s.\n", hostname);
    exit (EXIT_FAILURE);
  }
  name->sin_addr = *(struct in_addr *) hostinfo->h_addr;
}

int
setup_socket (int port)
{
  int sock;
  struct sockaddr_in server;

  /* Create the server socket */
  sock = socket (PF_INET, SOCK_STREAM, 0);
  if (sock < 0) {
    perror ("socket (listen)");
    exit (EXIT_FAILURE);
  }

  /* Bind the server socket and set it up to accept connections. */
  init_sockaddr (&server, "0.0.0.0", port);
  if (bind (sock, (struct sockaddr *) &server, sizeof (server)) < 0) {
    perror ("bind (listen)");
    exit (EXIT_FAILURE);
  }
  if (listen (sock, 1) < 0) {
    perror ("listen");
    exit (EXIT_FAILURE);
  }
  if (fcntl (sock, F_SETFL, O_NDELAY) < 0) {
    perror ("fcntl sock");
    exit (EXIT_FAILURE);
  }

  return sock;
}

int
poll_socket (int sock)
{
  int gdbsock;
  size_t size;

  size = sizeof (clientname);
  gdbsock = accept (sock, (struct sockaddr *) &clientname, &size);
  if ((gdbsock < 0) && (errno != EWOULDBLOCK)) {
    perror ("accept");
    exit (EXIT_FAILURE);
  }
  else if (gdbsock > 0) {
    if (fcntl (gdbsock, F_SETFL, O_NDELAY) < 0) {
      perror ("fcntl gdbsock");
      exit (EXIT_FAILURE);
    }
  }

  return gdbsock;
}

void
print_client_info (void)
{
  printf ("TCP connection from host %s, port %hd\n", inet_ntoa (clientname.sin_addr),
          ntohs (clientname.sin_port));
}

